unit MyShape;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, ExtCtrls, Controls, Dialogs, Forms, Menus;

type
  TCallBackFunc = procedure(Sender: TObject) of object;

  TMyShape = class(TShape)
  private
    inReposition:    Boolean;
    PopMenuCreated:  Boolean;
    oldPos :         TPoint;
  public
    Count: Integer;
    CallBackFunc:    TCallBackFunc;
    constructor Create(TheOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   ControlMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure   ControlMouseEnter(Sender: TObject);
    procedure   ControlMouseLeave(Sender: TObject);
    procedure   ControlDblClick(Sender: TObject);
    procedure   CreatePopupMenu();
    procedure   CreateColorDialog(Sender: TObject);
    procedure   CreateSettingsDialog(Sender: TObject);
    function    IsFloat(var s: string): boolean;
    function    IsInteger(var s: string): boolean;
  end;



implementation

var
  OptAreaCnt: Integer = 0;    // class variable for counting/indexing all instances of TOpticalAreas

constructor TMyShape.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  inc(OptAreaCnt);
  Name            := 'MyShape' + IntToStr(OptAreaCnt);
  PopMenuCreated  := false;
  ShowHint        := true;
  OnMouseDown     := @ControlMouseDown;
  OnMouseMove     := @ControlMouseMove;
  OnMouseUp       := @ControlMouseUp;
  OnMouseEnter    := @ControlMouseEnter;
  OnMouseLeave    := @ControlMouseLeave;
  OnDblClick      := @ControlDblClick;
end;

destructor  TMyShape.Destroy;
begin
  inherited Destroy;
end;

procedure TMyShape.ControlMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Sender is TMyShape) then
  begin
    if Button = mbLeft then
    begin
      inReposition := True;
      oldPos       := Point(X, Y);
    end
    else if Button = mbRight then
      CreatePopupMenu();
  end;
end; (*ControlMouseDown*)

procedure TMyShape.ControlMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
  minWidth = 5;
  minHeight = 5;
var
  newPos: TPoint;
  frmPoint : TPoint;
begin
  if inReposition then
  begin
    with TControl(Sender) do
    begin
      newPos := Point(X, Y);
      if ssShift in Shift then
      begin //resize
        Screen.Cursor := crSizeNWSE;
        frmPoint := Point(Width + X - oldPos.X, Height + Y - oldPos.Y);
        if frmPoint.X > minWidth then
          Width := frmPoint.X;
        if frmPoint.Y > minHeight then
          Height := frmPoint.Y;
        oldPos := newPos;
      end
      else //move
      begin
        Screen.Cursor := crSize;
        Left := Left - oldPos.X + newPos.X;
        Top := Top - oldPos.Y + newPos.Y;
      end;
    end;
  end;
  Hint := Name + ', Height: ' + IntToStr(Height) + ', Width: ' + IntToStr(Width);
end; (*ControlMouseMove*)

procedure TMyShape.ControlMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if inReposition then
  begin
    Screen.Cursor := crDefault;
    //ReleaseCapture;
    inReposition := False;
  end;
end; (*ControlMouseUp*)

procedure TMyShape.ControlMouseEnter(Sender: TObject);
begin
  //Repaint;
  Canvas.TextOut(1,1, 'Maus innerhalb');
end; (*ControlMouseEnter*)

procedure TMyShape.ControlMouseLeave(Sender: TObject);
begin
  //Repaint;
  Canvas.TextOut(1,1, 'Maus ausserhalb');
end; (*ControlMouseLeave*)

procedure TMyShape.ControlDblClick(Sender: TObject);
begin
  if TMyShape(Sender).ClassType = TMyShape then
  begin
    CallBackFunc(Sender);
    inReposition := False;
    TMyShape(Sender).Free;
    //Repaint;
  end;
end; (*ControlDblClick*)

procedure TMyShape.CreatePopupMenu();
var
  PopMenu:     TPopupMenu;
  NewItem:     TMenuItem;

begin
  if PopMenuCreated = false then
  begin
    PopMenu   := TPopupMenu.Create(Self);
    PopupMenu := PopMenu; // PopMenu dem Shape(aus TControl) zu weisen

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Color';
    NewItem.Name    := 'ChangeColor';
    NewItem.OnClick := @CreateColorDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Height';
    NewItem.Name    := 'Height';
    NewItem.OnClick := @CreateSettingsDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    NewItem         := TMenuItem.Create(Self);
    NewItem.Caption := 'Change Width';
    NewItem.Name    := 'Width';
    NewItem.OnClick := @CreateSettingsDialog; // Eventhandler zuweisen
    PopMenu.Items.Add(NewItem);

    PopMenuCreated  := true;
  end;
end;

procedure TMyShape.CreateColorDialog(Sender: TObject);
var
  ColorDialog: TColorDialog;

begin
  ColorDialog := TColorDialog.Create(Self);
  if ColorDialog.Execute then
    Brush.Color := (ColorDialog.Color);
end;

procedure TMyShape.CreateSettingsDialog(Sender: TObject);
var
  s: string;

begin
  if TMenuItem(Sender).Name = 'Height' then
  begin
    s := IntToStr(Height);
    while (not(InputQuery('Shape Settings', 'Height:', s)) or not(IsInteger(s))) do;
    Height := StrToInt(s);
  end;
  if TMenuItem(Sender).Name = 'Width' then
  begin
    s := IntToStr(Width);
    while (not(InputQuery('Shape Settings', 'Width:', s)) or not(IsInteger(s))) do;
    Width := StrToInt(s);
  end;

end;

function TMyShape.IsFloat(var s: string): boolean;
var
  float: real;
begin
// Prüfung ob Float
  if TryStrToFloat(s, float) then
    IsFloat := true
  else
    IsFloat := false;
end;

function TMyShape.IsInteger(var s: string): boolean;
var
  int: integer;
begin
// Prüfung ob Integer
  if TryStrToInt(s, int) then
    IsInteger := true
  else
    IsInteger := false;
end;

end.


